/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/dnn.h"
using namespace gctl;

int main(int argc, char const *argv[]) try
{
    convolution cnl_layer(0, 3, 10, 10, 3, 3, 2, 2, Valid, ReLU);
    std::clog << cnl_layer.layer_info() << std::endl;

    matrix<double> t(100, 3);
    std::clog << "T = \n";
    for (size_t c = 0; c < 3; c++)
    {
        for (size_t i = 0; i < 10; i++)
        {
            for (size_t j = 0; j < 10; j++)
            {
                t[i*10+j][c] = i*10 + j + 1;
                std::clog << t[i*10+j][c] << " ";
            }
            std::clog << "\n";
        }
    }

    array<double> weights(28);
    for (size_t i = 0; i < 28; i++)
    {
        weights[i] = 1.0;
    }
    
    cnl_layer.forward_propagation(weights, t);

    const matrix<double> &d = cnl_layer.forward_propagation_data();
    d.show(std::clog);
    return 0;
}
catch (std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}