/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_DNN_ACTIVATION_H
#define _GCTL_DNN_ACTIVATION_H

#include "gctl/core.h"
#include "gctl/io.h"
#include "gctl/algorithms.h"

#include "config.h"
#ifdef GCTL_AI_OPENMP
#include "omp.h"
#endif // GCTL_AI_OPENMP

namespace gctl
{
    enum activation_type_e
    {
        Identity,
        Mish,
        ReLU,
        PReLU,
        Sigmoid,
        SoftMax,
        Tanh,
        Unknown,
    };

    class dnn_activation
    {
    public:
        dnn_activation();
        virtual ~dnn_activation();

        virtual void activate(const matrix<double> &z, matrix<double> &a) = 0;
        virtual void apply_jacobian(const matrix<double> &z, const matrix<double> &a, 
            const matrix<double> &f, matrix<double> &g) = 0;
        virtual std::string activation_name() const;
        virtual activation_type_e activation_type() const;
    };
}



#endif // _GCTL_DNN_ACTIVATION_H