/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_DNN_HLAYER_CONVOLUTION_H
#define _GCTL_DNN_HLAYER_CONVOLUTION_H

#include "hlayer.h"

namespace gctl
{
    class convolution : public dnn_hlayer
    {
    public:
        convolution();
        convolution(int p_st, int channels, int in_rows, int in_cols, int filter_rows, int filter_cols, 
            int stride_rows, int stride_cols, pad_type_e pl_type, activation_type_e acti_type);
        virtual ~convolution();

        void forward_propagation(const array<double> &all_weights, const matrix<double> &prev_layer_data);
        void backward_propagation(const array<double> &all_weights, const array<double> &all_ders, 
            const matrix<double> &prev_layer_data, const matrix<double> &next_layer_data);

        hlayer_type_e get_layer_type() const;
        std::string get_layer_name() const;
        std::string layer_info() const;
        void save_layer_setup(std::ofstream &os) const;
        void load_layer_setup(std::ifstream &is);
        void save_weights2text(const array<double> &all_weights, std::ofstream &os) const;

    private:
        void init_convolution(int p_st, int channels, int in_rows, int in_cols, int filter_rows, int filter_cols, int stride_rows, 
            int stride_cols, pad_type_e pl_type, activation_type_e acti_type);
    
    private:
        int i_rows_, i_cols_, f_rows_, f_cols_, s_rows_, s_cols_;
        int o_rows_, o_cols_;
        int l_pad_, u_pad_;
        int chls_;
        int w_st_; // weight start index
        int b_st_; // bias start index

        array<int>  all_ders_count_;
        matrix<int> der_in_count_;

        array<int> p_idx_;
        pad_type_e p_type_;
    };
}

#endif // _GCTL_DNN_HLAYER_CONVOLUTION_H